@@ -816,6 +816,12 @@ def test_reproject_adaptive_roundtrip(file_format):
 
     header_out["DATE-OBS"] = header_out["DATE-OBS"].replace("T", " ")
 
+    # With sunpy 6.0.0 and later, additional keyword arguments are written out
+    # so we remove these as they are not important for the comparison with the
+    # reference files.
+    header_out.pop("DATE-AVG", None)
+    header_out.pop("MJD-AVG", None)
+
     return array_footprint_to_hdulist(output, footprint, header_out)
 
 
@@ -837,6 +843,12 @@ def test_reproject_adaptive_uncentered_jacobian():
 
     header_out["DATE-OBS"] = header_out["DATE-OBS"].replace("T", " ")
 
+    # With sunpy 6.0.0 and later, additional keyword arguments are written out
+    # so we remove these as they are not important for the comparison with the
+    # reference files.
+    header_out.pop("DATE-AVG", None)
+    header_out.pop("MJD-AVG", None)
+
     return array_footprint_to_hdulist(output, footprint, header_out)
 
 
@@ -1006,3 +1018,15 @@ def test_adaptive_input_output_types(
 
     assert_allclose(output_ref, output_test)
     assert_allclose(footprint_ref, footprint_test)
+
+
+def test_readonly_array():
+    wcs1 = WCS(naxis=2)
+    wcs2 = WCS(naxis=2)
+    wcs2.wcs.crpix = [2, 2]
+
+    array = np.random.random((128, 128))
+    array.flags.writeable = False
+
+    # This should work
+    reproject_adaptive((array, wcs1), wcs2, shape_out=(128, 128))
