@@ -1,3 +1,4 @@
+import logging
 import os
 import tempfile
 import uuid
@@ -5,6 +6,7 @@ import uuid
 import dask
 import dask.array as da
 import numpy as np
+from astropy.wcs import WCS
 from astropy.wcs.wcsapi import BaseHighLevelWCS, SlicedLowLevelWCS
 from astropy.wcs.wcsapi.high_level_wcs_wrapper import HighLevelWCSWrapper
 from dask import delayed
@@ -14,10 +16,29 @@ from .utils import _dask_to_numpy_memmap
 __all__ = ["_reproject_dispatcher"]
 
 
+class _ArrayContainer:
+    # When we set up as_delayed_memmap_path, if we pass a dask array to it,
+    # dask will actually compute the array before we get to the code inside
+    # as_delayed_memmap_path, so as a workaround we wrap any array we
+    # pass in using _ArrayContainer to make sure dask doesn't try and be smart.
+    def __init__(self, array):
+        self._array = array
+
+
 @delayed(pure=True)
 def as_delayed_memmap_path(array, tmp_dir):
+
+    # Extract array from _ArrayContainer
+    if isinstance(array, _ArrayContainer):
+        array = array._array
+    else:
+        raise TypeError("Expected _ArrayContainer in as_delayed_memmap_path")
+
+    logger = logging.getLogger(__name__)
     if isinstance(array, da.core.Array):
+        logger.info("Computing input dask array to Numpy memory-mapped array")
         array_path, _ = _dask_to_numpy_memmap(array, tmp_dir)
+        logger.info(f"Numpy memory-mapped array is now at {array_path}")
     else:
         array_path = os.path.join(tmp_dir, f"{uuid.uuid4()}.npy")
         array_memmapped = np.memmap(
@@ -81,18 +102,21 @@ def _reproject_dispatcher(
         An array in which to store the footprint of reprojected data.  This can be
         any numpy array including a memory map, which may be helpful when dealing with
         extremely large files.
-    parallel : bool or int, optional
+    parallel : bool or int or str, optional
         If `True`, the reprojection is carried out in parallel, and if a
-        positive integer, this specifies the number of processes to use.
+        positive integer, this specifies the number of threads to use.
         The reprojection will be parallelized over output array blocks specified
         by ``block_size`` (if the block size is not set, it will be determined
-        automatically).
+        automatically). To use the currently active dask scheduler (e.g.
+        dask.distributed), set this to ``'current-scheduler'``.
     reproject_func_kwargs : dict, optional
         Keyword arguments to pass through to ``reproject_func``
     return_type : {'numpy', 'dask'}, optional
         Whether to return numpy or dask arrays - defaults to 'numpy'.
     """
 
+    logger = logging.getLogger(__name__)
+
     if return_type is None:
         return_type = "numpy"
     elif return_type not in ("numpy", "dask"):
@@ -136,7 +160,11 @@ def _reproject_dispatcher(
                 )
 
             if isinstance(array_in, da.core.Array):
-                _, array_in = _dask_to_numpy_memmap(array_in, local_tmp_dir)
+                logger.info("Computing input dask array to Numpy memory-mapped array")
+                array_path, array_in = _dask_to_numpy_memmap(array_in, local_tmp_dir)
+                logger.info(f"Numpy memory-mapped array is now at {array_path}")
+
+            logger.info(f"Calling {reproject_func.__name__} in non-dask mode")
 
             try:
                 return reproject_func(
@@ -158,57 +186,82 @@ def _reproject_dispatcher(
 
         shape_in = array_in.shape
 
-        # When in parallel mode, we want to make sure we avoid having to copy the
-        # input array to all processes for each chunk, so instead we write out
-        # the input array to a Numpy memory map and load it in inside each process
-        # as a memory-mapped array. We need to be careful how this gets passed to
-        # reproject_single_block so we pass a variable that can be either a string
-        # or the array itself (for synchronous mode). If the input array is a dask
-        # array we should always write it out to a memmap even in synchronous mode
-        # otherwise map_blocks gets confused if it gets two dask arrays and tries
-        # to iterate over both.
+        # As we use the synchronous or threads scheduler, we don't need to worry about
+        # the data getting copied, so if the data is already a Numpy array (including
+        # a memory-mapped array) then we don't need to do anything special. However,
+        # if the input array is a dask array, we should convert it to a Numpy
+        # memory-mapped array so that it can be used by the various reprojection
+        # functions (which don't internally work with dask arrays).
 
-        if isinstance(array_in, da.core.Array) or parallel:
-            # If return_type=='dask',
+        if isinstance(array_in, np.memmap) and array_in.flags.c_contiguous:
+            array_in_or_path = array_in.filename, {
+                "dtype": array_in.dtype,
+                "shape": array_in.shape,
+                "offset": array_in.offset,
+            }
+        elif isinstance(array_in, da.core.Array) or return_type == "dask":
             if return_type == "dask":
                 # We should use a temporary directory that will persist beyond
                 # the call to the reproject function.
                 tmp_dir = tempfile.mkdtemp()
             else:
                 tmp_dir = local_tmp_dir
-            array_in_or_path = as_delayed_memmap_path(array_in, tmp_dir)
+            array_in_or_path = as_delayed_memmap_path(_ArrayContainer(array_in), tmp_dir)
         else:
-            # Here we could set array_in_or_path to array_in_path if it
-            # has been set previously, but in synchronous mode it is better to
-            # simply pass a reference to the memmap array itself to avoid having
-            # to load the memmap inside each reproject_single_block call.
+            # Here we could set array_in_or_path to array_in_path if it has
+            # been set previously, but in synchronous and threaded mode it is
+            # better to simply pass a reference to the memmap array itself to
+            # avoid having to load the memmap inside each
+            # reproject_single_block call.
             array_in_or_path = array_in
 
         def reproject_single_block(a, array_or_path, block_info=None):
-            if a.ndim == 0 or block_info is None or block_info == []:
+
+            if (
+                a.ndim == 0
+                or block_info is None
+                or block_info == []
+                or (isinstance(block_info, np.ndarray) and block_info.tolist() == [])
+            ):
                 return np.array([a, a])
-            slices = [slice(*x) for x in block_info[None]["array-location"][-wcs_out.pixel_n_dim :]]
+
+            # The WCS class from astropy is not thread-safe, see e.g.
+            # https://github.com/astropy/astropy/issues/16244
+            # https://github.com/astropy/astropy/issues/16245
+            # To work around these issues, we make sure we do a deep copy of
+            # the WCS object in here when using FITS WCS. This is a very fast
+            # operation (<0.1ms) so should not be a concern in terms of
+            # performance. We only need to do this for FITS WCS.
+
+            wcs_in_cp = wcs_in.deepcopy() if isinstance(wcs_in, WCS) else wcs_in
+            wcs_out_cp = wcs_out.deepcopy() if isinstance(wcs_out, WCS) else wcs_out
+
+            slices = [
+                slice(*x) for x in block_info[None]["array-location"][-wcs_out_cp.pixel_n_dim :]
+            ]
 
             if isinstance(wcs_out, BaseHighLevelWCS):
-                low_level_wcs = SlicedLowLevelWCS(wcs_out.low_level_wcs, slices=slices)
+                low_level_wcs = SlicedLowLevelWCS(wcs_out_cp.low_level_wcs, slices=slices)
             else:
-                low_level_wcs = SlicedLowLevelWCS(wcs_out, slices=slices)
+                low_level_wcs = SlicedLowLevelWCS(wcs_out_cp, slices=slices)
 
             wcs_out_sub = HighLevelWCSWrapper(low_level_wcs)
 
-            if isinstance(array_or_path, str):
+            if isinstance(array_or_path, tuple):
+                array_in = np.memmap(array_or_path[0], **array_or_path[1])
+            elif isinstance(array_or_path, str):
                 array_in = np.memmap(array_or_path, dtype=float, shape=shape_in)
             else:
                 array_in = array_or_path
 
             if array_or_path is None:
-                raise ValueError()
+                raise RuntimeError("array_or_path is not set")
 
             shape_out = block_info[None]["chunk-shape"][1:]
 
             array, footprint = reproject_func(
                 array_in,
-                wcs_in,
+                wcs_in_cp,
                 wcs_out_sub,
                 shape_out=shape_out,
                 array_out=np.zeros(shape_out),
@@ -239,7 +292,9 @@ def _reproject_dispatcher(
             else:
                 rechunk_kwargs = {}
             array_out_dask = da.empty(shape_out)
-            array_out_dask = array_out_dask.rechunk(block_size_limit=8 * 1024**2, **rechunk_kwargs)
+            array_out_dask = array_out_dask.rechunk(block_size_limit=64 * 1024**2, **rechunk_kwargs)
+
+        logger.info(f"Setting up output dask array with map_blocks")
 
         result = da.map_blocks(
             reproject_single_block,
@@ -267,27 +322,39 @@ def _reproject_dispatcher(
 
         if parallel:
             # As discussed in https://github.com/dask/dask/issues/9556, da.store
-            # will not work well in multiprocessing mode when the destination is a
+            # will not work well in parallel mode when the destination is a
             # Numpy array. Instead, in this case we save the dask array to a zarr
             # array on disk which can be done in parallel, and re-load it as a dask
             # array. We can then use da.store in the next step using the
             # 'synchronous' scheduler since that is I/O limited so does not need
             # to be done in parallel.
 
-            if isinstance(parallel, int):
-                if parallel > 0:
-                    workers = {"num_workers": parallel}
-                else:
-                    raise ValueError("The number of processors to use must be strictly positive")
-            else:
-                workers = {}
-
             zarr_path = os.path.join(local_tmp_dir, f"{uuid.uuid4()}.zarr")
 
-            with dask.config.set(scheduler="processes", **workers):
+            logger.info(f"Computing output array directly to zarr array at {zarr_path}")
+
+            if parallel == "current-scheduler":
+                # Just use whatever is the current active scheduler, which can
+                # be used for e.g. dask.distributed
                 result.to_zarr(zarr_path)
+            else:
+                if isinstance(parallel, bool):
+                    workers = {}
+                else:
+                    if parallel > 0:
+                        workers = {"num_workers": parallel}
+                    else:
+                        raise ValueError(
+                            "The number of processors to use must be strictly positive"
+                        )
+
+                with dask.config.set(scheduler="threads", **workers):
+                    result.to_zarr(zarr_path)
+
             result = da.from_zarr(zarr_path)
 
+        logger.info(f"Copying output zarr array into output Numpy arrays")
+
         if return_footprint:
             da.store(
                 [result[0], result[1]],
